@@ -17,7 +17,7 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <malloc.h>  // for alloca
 #endif
 
@@ -206,7 +206,7 @@ static void dec_process(MSFilter *f)
 	mblk_t *om;
 
 	while((im=ms_queue_get(f->inputs[0]))) {
-		int payloadlen = im->b_wptr - im->b_rptr;
+		int payloadlen = (int)(im->b_wptr - im->b_rptr);
 		int declen;
 
 		om=allocb(payloadlen*4,0);
