@@ -450,6 +450,10 @@ static void configure_audio_session (au_card_t* d,uint64_t time) {
 
 static bool_t  start_audio_unit (au_filter_base_t* d,uint64_t time) {
 	au_card_t* card=d->card;
+	if (card->io_unit == NULL) {
+		create_io_unit(&card->io_unit, card->ms_snd_card);
+		if (card->io_unit == NULL) ms_fatal("io_unit is NULL");
+	}
 	if (!card->io_unit_started && (card->last_failed_iounit_start_time == 0 || (time - card->last_failed_iounit_start_time)>100)) {
 
 		check_audiounit_call(AudioUnitInitialize(card->io_unit));
@@ -473,7 +477,7 @@ static bool_t  start_audio_unit (au_filter_base_t* d,uint64_t time) {
 									  , &quality
 									  , &qualitySize));
 
-		ms_error("I/O unit latency [%f], quality [%li]",delay,quality);
+		ms_message("I/O unit latency [%f], quality [%li]",delay,quality);
 		Float32 hwoutputlatency;
 		UInt32 hwoutputlatencySize=sizeof(hwoutputlatency);
 
@@ -534,9 +538,6 @@ static void stop_audio_unit (au_card_t* d) {
 	}
 	if (d->io_unit) {
 		check_audiounit_call( AudioUnitUninitialize(d->io_unit) );
-	}
-
-	if (d->io_unit) {
 		destroy_audio_unit(d);
 	}
 	d->rate=0; /*uninit*/
@@ -829,20 +830,20 @@ static void shutdown_timer(CFRunLoopTimerRef timer, void *info){
 static void check_unused(au_card_t *card){
 	if (card->read_data==NULL && card->write_data==NULL ){
 
-		if( !card->is_tester && card->shutdown_timer==NULL){
-		/*program the shutdown of the audio unit in a few seconds*/
-		CFRunLoopTimerContext ctx={0};
-		ctx.info=card;
-		card->shutdown_timer=CFRunLoopTimerCreate (
-											kCFAllocatorDefault,
-											CFAbsoluteTimeGetCurrent() + 2.5,
-											0,
-											0,
-											0,
-											shutdown_timer,
-											&ctx
-											);
-		CFRunLoopAddTimer(CFRunLoopGetMain(), card->shutdown_timer,kCFRunLoopCommonModes);
+		if (!card->is_tester && card->shutdown_timer==NULL){
+			/*program the shutdown of the audio unit in a few seconds*/
+			CFRunLoopTimerContext ctx={0};
+			ctx.info=card;
+			card->shutdown_timer=CFRunLoopTimerCreate (
+												kCFAllocatorDefault,
+												CFAbsoluteTimeGetCurrent() + 2.5,
+												0,
+												0,
+												0,
+												shutdown_timer,
+												&ctx
+												);
+			CFRunLoopAddTimer(CFRunLoopGetMain(), card->shutdown_timer,kCFRunLoopCommonModes);
 		} else if( card->is_tester ){
 			stop_audio_unit(card);
 		}
