@@ -26,7 +26,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "ortp/rtp.h"
 
 #include <stdint.h>
-#if defined(ANDROID) || (defined(WIN32) && !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP))
+#if defined(ANDROID) || defined(MS2_WINDOWS_PHONE) || defined(MS2_WINDOWS_UNIVERSAL)
 #include <opus.h>
 #else
 #include <opus/opus.h>
@@ -93,7 +93,7 @@ static void ms_opus_enc_init(MSFilter *f) {
 	d->ptime = 20;
 	d->minptime = 20; // defaut shall be 3, but we never use less than 20.
 	d->maxaveragebitrate = -1;
-	d->stereo = 1;
+	d->stereo = 0; /*stereo=0 is the default mode in opus rtp draft*/
 	d->channels = 1;
 	d->vbr = 1;
 	d->useinbandfec = 0;
@@ -104,6 +104,8 @@ static void ms_opus_enc_init(MSFilter *f) {
 
 static void ms_opus_enc_preprocess(MSFilter *f) {
 	int error;
+	int opusComplexity = -1;
+	const char *env = NULL;
 
 	OpusEncData *d = (OpusEncData *)f->data;
 	/* create the encoder */
@@ -113,14 +115,32 @@ static void ms_opus_enc_preprocess(MSFilter *f) {
 		return;
 	}
 
-	/* set complexity to 0 for single processor arm devices */
-#if defined(__arm__) || defined(_M_ARM)
-	if (ms_factory_get_cpu_count(f->factory)==1){
-		opus_encoder_ctl(d->state, OPUS_SET_COMPLEXITY(0));
-	}else{
-		opus_encoder_ctl(d->state, OPUS_SET_COMPLEXITY(5));
-	}
+	
+#ifndef MS2_WINDOWS_UNIVERSAL
+	env = getenv("MS2_OPUS_COMPLEXITY");
 #endif
+	if (env != NULL) {
+		opusComplexity = atoi(env);
+		if (opusComplexity < -1)
+			opusComplexity = -1; /*our default value*/
+		if (opusComplexity > 10)
+			opusComplexity = 10;
+	}
+	if (opusComplexity == -1){
+#if defined(__arm__) || defined(_M_ARM)
+		int cpucount = ms_factory_get_cpu_count(f->factory);
+		if (cpucount == 1){
+			opusComplexity = 0; /* set complexity to 0 for single processor arm devices */ 
+		}else if (cpucount == 2) {
+			opusComplexity = 5; 
+		}
+#endif
+	}
+	if (opusComplexity != -1){
+		ms_message("Set Opus complexity to %d", opusComplexity);
+		opus_encoder_ctl(d->state, OPUS_SET_COMPLEXITY(opusComplexity));
+	} /*otherwise we let opus with its default value, which is 9*/
+	
 	error = opus_encoder_ctl(d->state, OPUS_SET_PACKET_LOSS_PERC(10));
 	if (error != OPUS_OK) {
 		ms_error("Could not set default loss percentage to opus encoder: %s", opus_strerror(error));
@@ -136,6 +156,9 @@ static void ms_opus_enc_preprocess(MSFilter *f) {
 		if (error != OPUS_OK) {
 			ms_error("could not force mono channel to opus encoder: %s", opus_strerror(error));
 		}
+		if (d->channels == 2) ms_message("Opus encoder configured to encode mono despite it is feed with stereo.");
+	}else if (d->channels == 2){
+		ms_message("Opus encoder configured to encode stereo.");
 	}
 
 	ms_filter_lock(f);
@@ -280,7 +303,7 @@ static void compute_max_bitrate(OpusEncData *d, int ptimeStep) {
 	if (normalized_cbr<6000) {
 		int initial_value = normalized_cbr;
 		normalized_cbr = 6000;
-		d->max_network_bitrate = (normalized_cbr/(pps*8) + 12 + 8 + 20) *8*pps;
+		d->max_network_bitrate = (int)((normalized_cbr/(pps*8) + 12 + 8 + 20) *8*pps);
 		ms_warning("Opus encoder does not support bitrate [%i]. Instead set to 6kbps, network bitrate [%d]", initial_value, d->max_network_bitrate);
 	}
 	if (d->maxaveragebitrate>0)
@@ -289,7 +312,7 @@ static void compute_max_bitrate(OpusEncData *d, int ptimeStep) {
 	if (normalized_cbr>maxaveragebitrate) {
 		int initial_value = normalized_cbr;
 		normalized_cbr = maxaveragebitrate;
-		d->max_network_bitrate = (normalized_cbr/(pps*8) + 12 + 8 + 20) *8*pps;
+		d->max_network_bitrate = (int)((normalized_cbr/(pps*8) + 12 + 8 + 20) *8*pps);
 		ms_warning("Opus encoder cannot set codec bitrate to [%i] because of maxaveragebitrate constraint or absolute maximum bitrate value. New network bitrate is [%i]", initial_value, d->max_network_bitrate);
 	}
 	ms_message("MSOpusEnc: codec bitrate set to [%i] with ptime [%i]", normalized_cbr, d->ptime);
@@ -671,7 +694,7 @@ static void ms_opus_dec_process(MSFilter *f) {
 	while ((im = ms_queue_get(f->inputs[0])) != NULL) {
 		om = allocb(5760 * d->channels * SIGNAL_SAMPLE_SIZE, 0); /* 5760 is the maximum number of sample in a packet (120ms at 48KHz) */
 
-		frames = opus_decode(d->state, (const unsigned char *)im->b_rptr, im->b_wptr - im->b_rptr, (opus_int16 *)om->b_wptr, 5760, 0);
+		frames = opus_decode(d->state, (const unsigned char *)im->b_rptr, (opus_int32)(im->b_wptr - im->b_rptr), (opus_int16 *)om->b_wptr, 5760, 0);
 
 		if (frames < 0) {
 			ms_warning("Opus decoder error: %s", opus_strerror(frames));
@@ -797,6 +820,12 @@ static int ms_opus_dec_set_nchannels(MSFilter *f, void *arg) {
 	return 0;
 }
 
+static int ms_opus_dec_get_nchannels(MSFilter *f, void *arg) {
+	OpusDecData *d = (OpusDecData *)f->data;
+	*(int*)arg=d->channels;
+	return 0;
+}
+
 static MSFilterMethod ms_opus_dec_methods[] = {
 	{	MS_FILTER_SET_SAMPLE_RATE,	ms_opus_dec_set_sample_rate	},
 	{	MS_FILTER_GET_SAMPLE_RATE,	ms_opus_dec_get_sample_rate	},
@@ -805,6 +834,7 @@ static MSFilterMethod ms_opus_dec_methods[] = {
 	{	MS_AUDIO_DECODER_SET_RTP_PAYLOAD_PICKER, ms_opus_set_rtp_picker },
 	{ 	MS_AUDIO_DECODER_HAVE_PLC,		ms_opus_dec_have_plc	},
 	{	MS_FILTER_SET_NCHANNELS,	ms_opus_dec_set_nchannels	},
+	{	MS_FILTER_GET_NCHANNELS,	ms_opus_dec_get_nchannels	},
 	{	0,				NULL				}
 };
 
