@@ -34,7 +34,7 @@
 #undef PACKAGE_TARNAME
 #undef PACKAGE_VERSION
 
-#if defined(ANDROID) || !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+#if defined(ANDROID) || defined(MS2_WINDOWS_PHONE) || defined(MS2_WINDOWS_UNIVERSAL)
 // Android and Windows phone don't use make install
 #include <srtp.h>
 //#include <srtp_priv.h>
@@ -51,12 +51,12 @@
 typedef struct _MSSrtpStreamContext {
 	srtp_t srtp;
 	RtpTransportModifier *modifier;
+	ms_mutex_t mutex;
 	bool_t secured;
 	bool_t mandatory_enabled;
 	bool_t is_rtp;
-	ms_mutex_t mutex;
-
 } MSSrtpStreamContext;
+
 struct _MSSrtpCtx {
 	MSSrtpStreamContext send_rtp_context;
 	MSSrtpStreamContext send_rtcp_context;
@@ -107,12 +107,12 @@ static int _process_on_send(RtpSession* session,MSSrtpStreamContext *ctx, mblk_t
 	bool_t is_rtp=ctx->is_rtp;
 	rtp_header_t *rtp_header=is_rtp?(rtp_header_t*)m->b_rptr:NULL;
 	rtcp_common_header_t *rtcp_header=!is_rtp?(rtcp_common_header_t*)m->b_rptr:NULL;
-	slen=msgdsize(m);
+	slen=(int)msgdsize(m);
 
 	if (rtp_header && (slen>RTP_FIXED_HEADER_SIZE && rtp_header->version==2)) {
 		ms_mutex_lock(&ctx->mutex);
 		if (!ctx->secured) {
-			/*does not make sens to protec, because we don't have any key*/
+			/*does not make sense to protect, because we don't have any key*/
 			err=err_status_ok;
 			slen = 0; /*droping packets*/
 		} else {
@@ -125,7 +125,7 @@ static int _process_on_send(RtpSession* session,MSSrtpStreamContext *ctx, mblk_t
 		ms_mutex_lock(&ctx->mutex);
 		if (!ctx->secured) {
 			err=err_status_ok;
-			/*does not make sens to protec, because we don't have any key*/
+			/*does not make sense to protect, because we don't have any key*/
 			slen = 0; /*droping packets*/
 		} else {
 		/* defragment incoming message and enlarge the buffer for srtp to write its data */
@@ -151,7 +151,7 @@ static int ms_srtp_process_on_send(RtpTransportModifier *t, mblk_t *m){
 }
 
 static int ms_srtp_process_dummy(RtpTransportModifier *t, mblk_t *m) {
-	return msgdsize(m);
+	return (int)msgdsize(m);
 }
 static int _process_on_receive(RtpSession* session,MSSrtpStreamContext *ctx, mblk_t *m, int err){
 	int slen;
@@ -179,7 +179,7 @@ static int _process_on_receive(RtpSession* session,MSSrtpStreamContext *ctx, mbl
 	}
 }
 static int ms_srtp_process_on_receive(RtpTransportModifier *t, mblk_t *m){
-	return _process_on_receive(t->session,(MSSrtpStreamContext*)t->data, m,msgdsize(m));
+	return _process_on_receive(t->session,(MSSrtpStreamContext*)t->data, m,(int)msgdsize(m));
 }
 
 /**** Session management functions ****/
@@ -192,7 +192,7 @@ static void ms_srtp_transport_modifier_destroy(RtpTransportModifier *tp){
 	ms_free(tp);
 }
 
-static MSSrtpStreamContext* get_stream_context(struct _MSMediaStreamSessions *sessions, bool_t is_send, bool_t is_rtp) {
+static MSSrtpStreamContext* get_stream_context(MSMediaStreamSessions *sessions, bool_t is_send, bool_t is_rtp) {
 	if (is_send && is_rtp)
 		return &sessions->srtp_context->send_rtp_context;
 	else if (is_send && !is_rtp)
@@ -204,7 +204,7 @@ static MSSrtpStreamContext* get_stream_context(struct _MSMediaStreamSessions *se
 }
 
 
-static int ms_media_stream_session_fill_srtp_context(struct _MSMediaStreamSessions *sessions, bool_t is_send, bool_t is_rtp) {
+static int ms_media_stream_session_fill_srtp_context(MSMediaStreamSessions *sessions, bool_t is_send, bool_t is_rtp) {
 	err_status_t err=0;
 	RtpTransport *transport=NULL;
 	MSSrtpStreamContext* stream_ctx = get_stream_context(sessions,is_send,is_rtp);
@@ -244,7 +244,7 @@ end:
 	return err;
 
 }
-int ms_media_stream_session_fill_srtp_context_all_stream(struct _MSMediaStreamSessions *sessions) {
+int ms_media_stream_sessions_fill_srtp_context_all_stream(struct _MSMediaStreamSessions *sessions) {
 	int  err = -1;
 	/*check if exist before filling*/
 
@@ -309,12 +309,11 @@ static int ms_add_srtp_stream(srtp_t srtp, MSCryptoSuite suite, uint32_t ssrc, c
 			return -1;
 		}
 	}else {
-		if (key_length != policy.rtcp.cipher_key_len) {
-			ms_error("Key size (%i) doesn't match the selected srtp profile (required %d)", (int)key_length, policy.rtcp.cipher_key_len);	
+		if (ms_set_srtp_crypto_policy(suite, &policy.rtcp) != 0) {
 			return -1;
 		}
-
-		if (ms_set_srtp_crypto_policy(suite, &policy.rtcp) != 0) {
+		if (key_length != policy.rtcp.cipher_key_len) {
+			ms_error("Key size (%i) doesn't match the selected srtp profile (required %d)", (int)key_length, policy.rtcp.cipher_key_len);	
 			return -1;
 		}
 	}
@@ -369,7 +368,7 @@ void ms_srtp_shutdown(void){
 	}
 }
 
-static int media_stream_set_srtp_key_base(struct _MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, bool_t is_send, bool_t is_rtp){
+static int ms_media_stream_sessions_set_srtp_key_base(MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, bool_t is_send, bool_t is_rtp){
 	MSSrtpStreamContext* stream_ctx;
 	uint32_t ssrc;
 	int error = -1;
@@ -384,7 +383,7 @@ static int media_stream_set_srtp_key_base(struct _MSMediaStreamSessions *session
 		return error;
 	}
 
-	if ((error = ms_add_srtp_stream(stream_ctx->srtp,suite, ssrc, key, key_length, is_send, TRUE))) {
+	if ((error = ms_add_srtp_stream(stream_ctx->srtp,suite, ssrc, key, key_length, is_send, is_rtp))) {
 		stream_ctx->secured=FALSE;
 		return error;
 	}
@@ -393,15 +392,15 @@ static int media_stream_set_srtp_key_base(struct _MSMediaStreamSessions *session
 	return 0;
 }
 
-static int media_stream_set_srtp_key(struct _MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, bool_t is_send, MSSrtpStreamType stream_type){
+static int ms_media_stream_sessions_set_srtp_key(MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, bool_t is_send, MSSrtpStreamType stream_type){
 	int error = 0;
 
 	if (stream_type == MSSRTP_ALL_STREAMS || stream_type == MSSRTP_RTP_STREAM) {
-		error=media_stream_set_srtp_key_base(sessions,suite,key,key_length,is_send,TRUE);
+		error=ms_media_stream_sessions_set_srtp_key_base(sessions,suite,key,key_length,is_send,TRUE);
 	}
 
 	if (error == 0 && (stream_type == MSSRTP_ALL_STREAMS || stream_type == MSSRTP_RTCP_STREAM)) {
-		error=media_stream_set_srtp_key_base(sessions,suite,key,key_length,is_send,FALSE);
+		error=ms_media_stream_sessions_set_srtp_key_base(sessions,suite,key,key_length,is_send,FALSE);
 	}
 	return error;
 }
@@ -414,7 +413,7 @@ bool_t ms_srtp_supported(void){
 }
 
 
-int media_stream_set_srtp_recv_key_b64(struct _MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* b64_key){
+int ms_media_stream_sessions_set_srtp_recv_key_b64(MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* b64_key){
 	int retval;
 
 	/* decode b64 key */
@@ -428,14 +427,14 @@ int media_stream_set_srtp_recv_key_b64(struct _MSMediaStreamSessions *sessions,
 	}
 
 	/* pass decoded key to set_recv_key function */
-	retval = media_stream_set_srtp_recv_key(sessions, suite, key, key_length, MSSRTP_ALL_STREAMS);
+	retval = ms_media_stream_sessions_set_srtp_recv_key(sessions, suite, key, key_length, MSSRTP_ALL_STREAMS);
 
 	ms_free(key);
 
 	return retval;
 }
 
-bool_t media_stream_session_secured(const struct _MSMediaStreamSessions *sessions,MediaStreamDir dir) {
+bool_t ms_media_stream_sessions_secured(const MSMediaStreamSessions *sessions,MediaStreamDir dir) {
 	if(!sessions->srtp_context)
 		return FALSE;
 
@@ -450,10 +449,11 @@ bool_t media_stream_session_secured(const struct _MSMediaStreamSessions *session
 }
 
 
-int media_stream_set_srtp_recv_key(struct _MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, MSSrtpStreamType stream_type) {
-	return media_stream_set_srtp_key(sessions,suite,key,key_length,FALSE,stream_type);
+int ms_media_stream_sessions_set_srtp_recv_key(MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, MSSrtpStreamType stream_type) {
+	return ms_media_stream_sessions_set_srtp_key(sessions,suite,key,key_length,FALSE,stream_type);
 }
-int media_stream_set_srtp_send_key_b64(struct _MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* b64_key){
+
+int ms_media_stream_sessions_set_srtp_send_key_b64(MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* b64_key){
 	int retval;
 
 	/* decode b64 key */
@@ -467,24 +467,24 @@ int media_stream_set_srtp_send_key_b64(struct _MSMediaStreamSessions *sessions,
 	}
 
 	/* pass decoded key to set_send_key function */
-	retval = media_stream_set_srtp_send_key(sessions, suite, key, key_length, MSSRTP_ALL_STREAMS);
+	retval = ms_media_stream_sessions_set_srtp_send_key(sessions, suite, key, key_length, MSSRTP_ALL_STREAMS);
 
 	ms_free(key);
 
 	return retval;
 }
 
-int media_stream_set_srtp_send_key(struct _MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, MSSrtpStreamType stream_type){
-	return media_stream_set_srtp_key(sessions,suite,key,key_length,TRUE,stream_type);
+int ms_media_stream_sessions_set_srtp_send_key(MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, MSSrtpStreamType stream_type){
+	return ms_media_stream_sessions_set_srtp_key(sessions,suite,key,key_length,TRUE,stream_type);
 }
 
 
-int media_stream_session_encryption_mandatory_enable(struct _MSMediaStreamSessions *sessions, bool_t yesno) {
-	/*for now, managing all stream in one time*/
+int ms_media_stream_sessions_set_encryption_mandatory(MSMediaStreamSessions *sessions, bool_t yesno) {
+	/*for now, managing all streams in one time*/
 	int err;
 	check_and_create_srtp_context(sessions);
 	if (yesno) {
-		if ((err = ms_media_stream_session_fill_srtp_context_all_stream(sessions))) {
+		if ((err = ms_media_stream_sessions_fill_srtp_context_all_stream(sessions))) {
 			return err;;
 		}
 	}
@@ -495,7 +495,7 @@ int media_stream_session_encryption_mandatory_enable(struct _MSMediaStreamSessio
 	return 0;
 }
 
-bool_t media_stream_session_encryption_mandatory_enabled(const struct _MSMediaStreamSessions *sessions) {
+bool_t ms_media_stream_sessions_get_encryption_mandatory(const MSMediaStreamSessions *sessions) {
 
 	if(!sessions->srtp_context)
 		return FALSE;
@@ -522,40 +522,48 @@ int ms_srtp_init(void) {
 void ms_srtp_shutdown(void){
 }
 
-int media_stream_set_srtp_recv_key_b64(struct _MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* b64_key){
+int ms_media_stream_sessions_set_srtp_recv_key_b64(struct _MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* b64_key){
 	ms_error("Unable to set srtp recv key b64: srtp support disabled in mediastreamer2");
 	return -1;
 }
 
-int media_stream_set_srtp_recv_key(struct _MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, MSSrtpStreamType stream_type){
+int ms_media_stream_sessions_set_srtp_recv_key(MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, MSSrtpStreamType stream_type){
 	ms_error("Unable to set srtp recv key: srtp support disabled in mediastreamer2");
 	return -1;
 }
 
-int media_stream_set_srtp_send_key_b64(struct _MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* b64_key){
+int ms_media_stream_sessions_set_srtp_send_key_b64(MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* b64_key){
 	ms_error("Unable to set srtp send key b64: srtp support disabled in mediastreamer2");
 	return -1;
 }
 
-int media_stream_set_srtp_send_key(struct _MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, MSSrtpStreamType stream_type){
+int ms_media_stream_sessions_set_srtp_send_key(MSMediaStreamSessions *sessions, MSCryptoSuite suite, const char* key, size_t key_length, MSSrtpStreamType stream_type){
 	ms_error("Unable to set srtp send key: srtp support disabled in mediastreamer2");
 	return -1;
 }
 
-int ms_srtp_dealloc(MSSrtpCtx session)
-{
-	return -1;
-}
-int media_stream_session_set_encryption_mandatory(struct _MSMediaStreamSessions *sessions, bool_t yesno) {
+int media_stream_session_set_encryption_mandatory(MSMediaStreamSessions *sessions, bool_t yesno) {
 	ms_error("Unable to set srtp encryption mandatory mode: srtp support disabled in mediastreamer2");
 	return -1;
 }
 
-bool_t media_stream_session_get_encryption_mandatory(const struct _MSMediaStreamSessions *sessions) {
+bool_t media_stream_session_get_encryption_mandatory(const MSMediaStreamSessions *sessions) {
 	ms_error("Unable to get srtp encryption mandatory mode: srtp support disabled in mediastreamer2");
 	return -1;
 }
 
+bool_t ms_media_stream_sessions_secured(const MSMediaStreamSessions *sessions,MediaStreamDir dir) {
+	return FALSE;
+}
+
+void ms_srtp_context_delete(MSSrtpCtx* session) {
+	ms_error("Unable to delete srtp context [%p]: srtp support disabled in mediastreamer2",session);
+}
+
+int ms_media_stream_sessions_set_encryption_mandatory(MSMediaStreamSessions *sessions, bool_t yesno) {
+	ms_error("Unable to set encryption_mandatory [%p]: srtp support disabled in mediastreamer2",sessions);
+	return -1;
+}
 #endif
 
 const char * ms_srtp_stream_type_to_string(const MSSrtpStreamType type) {
