@@ -160,7 +160,7 @@ static void enc_preprocess(MSFilter *f) {
 		ms_error("Failed to get config: %s", vpx_codec_err_to_string(res));
 		return;
 	}
-	s->cfg.rc_target_bitrate = ((float)s->vconf.required_bitrate) * 0.92 / 1024.0; //0.9=take into account IP/UDP/RTP overhead, in average.
+	s->cfg.rc_target_bitrate = (unsigned int)(((float)s->vconf.required_bitrate) * 0.92f / 1024.0f); //0.92=take into account IP/UDP/RTP overhead, in average.
 	s->cfg.g_pass = VPX_RC_ONE_PASS; /* -p 1 */
 	s->cfg.g_timebase.num = 1;
 	s->cfg.g_timebase.den = (int)s->vconf.fps;
@@ -181,8 +181,8 @@ static void enc_preprocess(MSFilter *f) {
 	s->cfg.g_error_resilient = VPX_ERROR_RESILIENT_DEFAULT|VPX_ERROR_RESILIENT_PARTITIONS;
 	s->cfg.g_lag_in_frames = 0;
 
-	
-#if defined(ANDROID) || TARGET_OS_IPHONE==1
+
+#if defined(ANDROID) || (TARGET_OS_IPHONE == 1) || defined(__arm__) || defined(_M_ARM)
 	cpuused = 10 - s->cfg.g_threads; /*cpu/quality tradeoff: positive values decrease CPU usage at the expense of quality*/
 	if (cpuused < 7) cpuused = 7; /*values beneath 7 consume too much CPU*/
 	if( s->cfg.g_threads == 1 ){
@@ -248,7 +248,7 @@ static uint8_t enc_get_type_of_reference_frame_to_generate(EncState *s) {
 	}
 }
 
-static void check_most_recent(EncFrameState **found, vpx_codec_pts_t *most_recent, vpx_ref_frame_type_t type, 
+static void check_most_recent(EncFrameState **found, vpx_codec_pts_t *most_recent, vpx_ref_frame_type_t type,
 			      EncFrameState *state, bool_t is_acknoledged){
 	if (is_acknoledged && !state->acknowledged) return;
 	if (*most_recent<state->count){
@@ -304,7 +304,7 @@ static void enc_mark_reference_frame_as_sent(EncState *s, vpx_ref_frame_type_t f
 static bool_t is_reference_sane(EncState *s, EncFrameState *fs){
 	int diff=fs->picture_id-s->last_sli_id;
 	bool_t ret;
-	
+
 	if (!fs->is_independant && diff<enc_get_ref_frames_interval(s)){
 		ret=FALSE;
 	}else ret=TRUE;
@@ -416,7 +416,7 @@ static void enc_fill_encoder_flags(EncState *s, unsigned int *flags) {
 
 static bool_t is_frame_independent(unsigned int flags){
 	if (flags & VPX_EFLAG_FORCE_KF) return TRUE;
-	
+
 	if ((flags & VP8_EFLAG_FORCE_GF) || (flags & VP8_EFLAG_FORCE_ARF)){
 		if ((flags & VP8_EFLAG_NO_REF_ARF) && (flags & VP8_EFLAG_NO_REF_LAST) && (flags & VP8_EFLAG_NO_REF_GF))
 			return TRUE;
@@ -427,11 +427,11 @@ static bool_t is_frame_independent(unsigned int flags){
 static void enc_process(MSFilter *f) {
 	mblk_t *im;
 	uint64_t timems = f->ticker->time;
-	uint32_t timestamp = timems*90;
+	uint32_t timestamp = (uint32_t)(timems*90);
 	EncState *s = (EncState *)f->data;
 	unsigned int flags = 0;
 	vpx_codec_err_t err;
-	YuvBuf yuv;
+	MSPicture yuv;
 	bool_t is_ref_frame=FALSE;
 
 	ms_filter_lock(f);
@@ -448,7 +448,7 @@ static void enc_process(MSFilter *f) {
 
 	if ((im = ms_queue_peek_last(f->inputs[0])) != NULL) {
 		vpx_image_t img;
-		
+
 		flags = 0;
 		ms_yuv_buf_init_from_mblk(&yuv, im);
 		vpx_img_wrap(&img, VPX_IMG_FMT_I420, s->vconf.vsize.width, s->vconf.vsize.height, 1, yuv.planes[0]);
@@ -502,12 +502,12 @@ static void enc_process(MSFilter *f) {
 			if (is_frame_independent(flags)){
 				s->frames_state.last_independent_frame=s->frame_count;
 			}
-			
+
 			/* Pack the encoded frame. */
 			while( (pkt = vpx_codec_get_cx_data(&s->codec, &iter)) ) {
 				if ((pkt->kind == VPX_CODEC_CX_FRAME_PKT) && (pkt->data.frame.sz > 0)) {
 					Vp8RtpFmtPacket *packet = ms_new0(Vp8RtpFmtPacket, 1);
-					
+
 					packet->m = allocb(pkt->data.frame.sz, 0);
 					memcpy(packet->m->b_wptr, pkt->data.frame.buf, pkt->data.frame.sz);
 					packet->m->b_wptr += pkt->data.frame.sz;
@@ -535,10 +535,10 @@ static void enc_process(MSFilter *f) {
 					list = ms_list_append(list, packet);
 				}
 			}
-			
+
 #ifdef AVPF_DEBUG
 			ms_message("VP8 encoder picture_id=%i ***| %s | %s | %s | %s", (int)s->picture_id,
-				(flags & VPX_EFLAG_FORCE_KF) ? "KF " : (flags & VP8_EFLAG_FORCE_GF) ? "GF " :  (flags & VP8_EFLAG_FORCE_ARF) ? "ARF" : "   ",  
+				(flags & VPX_EFLAG_FORCE_KF) ? "KF " : (flags & VP8_EFLAG_FORCE_GF) ? "GF " :  (flags & VP8_EFLAG_FORCE_ARF) ? "ARF" : "   ",
 				(flags & VP8_EFLAG_NO_REF_GF) ? "NOREFGF" : "       ",
 				(flags & VP8_EFLAG_NO_REF_ARF) ? "NOREFARF" : "        ",
 				(flags & VP8_EFLAG_NO_REF_LAST) ? "NOREFLAST" : "         ");
@@ -581,7 +581,7 @@ static int enc_set_configuration(MSFilter *f, void *data) {
 
 	if (s->vconf.required_bitrate > s->vconf.bitrate_limit)
 		s->vconf.required_bitrate = s->vconf.bitrate_limit;
-	s->cfg.rc_target_bitrate = ((float)s->vconf.required_bitrate) * 0.92 / 1024.0; //0.9=take into account IP/UDP/RTP overhead, in average.
+	s->cfg.rc_target_bitrate = (unsigned int)(((float)s->vconf.required_bitrate) * 0.92f / 1024.0f); //0.92=take into account IP/UDP/RTP overhead, in average.
 	if (s->ready) {
 		ms_filter_lock(f);
 		enc_postprocess(f);
@@ -668,9 +668,9 @@ static bool_t should_generate_key_frame(EncState *s, int min_interval){
 
 static int enc_notify_pli(MSFilter *f, void *data) {
 	EncState *s = (EncState *)f->data;
-	ms_message("PLI requested");
+	ms_message("VP8: PLI requested");
 	if (should_generate_key_frame(s,MIN_KEY_FRAME_DIST)){
-		ms_message("PLI accepted");
+		ms_message("VP8: PLI accepted");
 		if (s->avpf_enabled == TRUE) {
 			s->invalid_frame_reported = TRUE;
 		} else {
@@ -699,11 +699,11 @@ static int enc_notify_sli(MSFilter *f, void *data) {
 	EncFrameState *fs;
 	int diff;
 	int most_recent;
-	
+
 	ms_filter_lock(f);
 	/* extend the SLI received picture-id (6 bits) to a normal picture id*/
 	most_recent=s->picture_id;
-	
+
 	diff=(64 + (int)(most_recent & 0x3F) - (int)(sli->picture_id & 0x3F)) % 64;
 	s->last_sli_id=most_recent-diff;
 	fs=enc_get_most_recent_reference_frame(s,FALSE);
@@ -758,6 +758,17 @@ static int enc_get_configuration_list(MSFilter *f, void *data) {
 	return 0;
 }
 
+static int enc_set_configuration_list(MSFilter *f, void *data) {
+	EncState *s = (EncState *)f->data;
+	const MSVideoConfiguration **vconf_list = (const MSVideoConfiguration **)data;
+	if (*vconf_list == NULL) {
+		s->vconf_list = &vp8_conf_list[0];
+	} else {
+		s->vconf_list = *vconf_list;
+	}
+	return 0;
+}
+
 static int enc_enable_avpf(MSFilter *f, void *data) {
 	EncState *s = (EncState *)f->data;
 	s->avpf_enabled = *((bool_t *)data) ? TRUE : FALSE;
@@ -778,6 +789,7 @@ static MSFilterMethod enc_methods[] = {
 	{ MS_VIDEO_ENCODER_NOTIFY_SLI,             enc_notify_sli             },
 	{ MS_VIDEO_ENCODER_NOTIFY_RPSI,            enc_notify_rpsi            },
 	{ MS_VIDEO_ENCODER_GET_CONFIGURATION_LIST, enc_get_configuration_list },
+	{ MS_VIDEO_ENCODER_SET_CONFIGURATION_LIST, enc_set_configuration_list },
 	{ MS_VIDEO_ENCODER_SET_CONFIGURATION,      enc_set_configuration      },
 	{ MS_VIDEO_ENCODER_ENABLE_AVPF,            enc_enable_avpf            },
 	{ 0,                                       NULL                       }
@@ -843,7 +855,6 @@ typedef struct DecState {
 	vpx_codec_iface_t *iface;
 	vpx_codec_flags_t flags;
 	Vp8RtpFmtUnpackerCtx unpacker;
-	mblk_t *curframe;
 	long last_cseq; /*last receive sequence number, used to locate missing partition fragment*/
 	int current_partition_id; /*current partition id*/
 	uint64_t last_error_reported_time;
@@ -865,7 +876,6 @@ static void dec_init(MSFilter *f) {
 	s->iface = vpx_codec_vp8_dx();
 	ms_message("Using %s", vpx_codec_iface_name(s->iface));
 
-	s->curframe = NULL;
 	s->last_error_reported_time = 0;
 	s->yuv_width = 0;
 	s->yuv_height = 0;
@@ -885,17 +895,21 @@ static void dec_preprocess(MSFilter* f) {
 	/* Initialize codec */
 	if (!s->ready){
 		s->flags = 0;
+#if 0
+		/* Deactivate fragments input for the vpx decoder because it has been broken in libvpx 1.4.
+		 * This will have the effect to output complete frames. */
 		if ((s->avpf_enabled == TRUE) && (caps & VPX_CODEC_CAP_INPUT_FRAGMENTS)) {
 			s->flags |= VPX_CODEC_USE_INPUT_FRAGMENTS;
 		}
+#endif
 		if (caps & VPX_CODEC_CAP_ERROR_CONCEALMENT) {
 			s->flags |= VPX_CODEC_USE_ERROR_CONCEALMENT;
 		}
-	#ifdef VPX_CODEC_CAP_FRAME_THREADING
+#ifdef VPX_CODEC_CAP_FRAME_THREADING
 		if ((caps & VPX_CODEC_CAP_FRAME_THREADING) && (ms_factory_get_cpu_count(f->factory) > 1)) {
 			s->flags |= VPX_CODEC_USE_FRAME_THREADING;
 		}
-	#endif
+#endif
 		if(vpx_codec_dec_init(&s->codec, s->iface, NULL, s->flags))
 			ms_error("Failed to initialize decoder");
 		ms_message("VP8: initializing decoder context: avpf=[%i] freeze_on_error=[%i]",s->avpf_enabled,s->freeze_on_error);
@@ -903,14 +917,13 @@ static void dec_preprocess(MSFilter* f) {
 		s->first_image_decoded = FALSE;
 		s->ready=TRUE;
 	}
-	
+
 }
 
 static void dec_uninit(MSFilter *f) {
 	DecState *s = (DecState *)f->data;
 	vp8rtpfmt_unpacker_uninit(&s->unpacker);
 	vpx_codec_destroy(&s->codec);
-	if (s->curframe != NULL) freemsg(s->curframe);
 	if (s->yuv_msg) freemsg(s->yuv_msg);
 	ms_queue_flush(&s->q);
 	ms_free(s);
@@ -922,63 +935,78 @@ static void dec_process(MSFilter *f) {
 	vpx_codec_err_t err;
 	vpx_image_t *img;
 	vpx_codec_iter_t iter = NULL;
+	MSQueue frame;
 	MSQueue mtofree_queue;
+	Vp8RtpFmtFrameInfo frame_info;
+	
+	ms_filter_lock(f);
 
+	ms_queue_init(&frame);
 	ms_queue_init(&mtofree_queue);
 
 	/* Unpack RTP payload format for VP8. */
-	vp8rtpfmt_unpacker_process(&s->unpacker, f->inputs[0]);
+	vp8rtpfmt_unpacker_feed(&s->unpacker, f->inputs[0]);
 
-	/* Decode unpacked VP8 partitions. */
-	while ((im = ms_queue_get(f->inputs[0])) != NULL) {
-		err = vpx_codec_decode(&s->codec, im->b_rptr, im->b_wptr - im->b_rptr, NULL, 0);
-		if ((s->flags & VPX_CODEC_USE_INPUT_FRAGMENTS) && (!err && mblk_get_marker_info(im))) {
-			err = vpx_codec_decode(&s->codec, NULL, 0, NULL, 0);
-		}
-		if (err) {
-			ms_warning("vp8 decode failed : %d %s (%s)\n", err, vpx_codec_err_to_string(err), vpx_codec_error_detail(&s->codec));
-		}
-		ms_queue_put(&mtofree_queue, im);
-	}
-
-	/* browse decoded frames */
-	while ((img = vpx_codec_get_frame(&s->codec, &iter))) {
-		int i, j;
-
-		if (s->yuv_width != img->d_w || s->yuv_height != img->d_h) {
-			if (s->yuv_msg) freemsg(s->yuv_msg);
-			s->yuv_msg = ms_yuv_buf_alloc(&s->outbuf, img->d_w, img->d_h);
-			s->yuv_width = img->d_w;
-			s->yuv_height = img->d_h;
-			ms_filter_notify_no_arg(f,MS_FILTER_OUTPUT_FMT_CHANGED);
+	/* Decode unpacked VP8 frames. */
+	while (vp8rtpfmt_unpacker_get_frame(&s->unpacker, &frame, &frame_info) == 0) {
+		while ((im = ms_queue_get(&frame)) != NULL) {
+			err = vpx_codec_decode(&s->codec, im->b_rptr, (unsigned int)(im->b_wptr - im->b_rptr), NULL, 0);
+			if ((s->flags & VPX_CODEC_USE_INPUT_FRAGMENTS) && mblk_get_marker_info(im)) {
+				err = vpx_codec_decode(&s->codec, NULL, 0, NULL, 0);
+			}
+			if (err) {
+				ms_warning("vp8 decode failed : %d %s (%s)\n", err, vpx_codec_err_to_string(err), vpx_codec_error_detail(&s->codec)?vpx_codec_error_detail(&s->codec):"no details");
+			}
+			ms_queue_put(&mtofree_queue, im);
 		}
 
-		/* scale/copy frame to destination mblk_t */
-		for (i = 0; i < 3; i++) {
-			uint8_t *dest = s->outbuf.planes[i];
-			uint8_t *src = img->planes[i];
-			int h = img->d_h >> ((i > 0) ? 1 : 0);
+		/* Get decoded frame */
+		if ((img = vpx_codec_get_frame(&s->codec, &iter))) {
+			int i, j;
+			int reference_updates = 0;
 
-			for (j = 0; j < h; j++) {
-				memcpy(dest, src, s->outbuf.strides[i]);
-				dest += s->outbuf.strides[i];
-				src += img->stride[i];
+			if (vpx_codec_control(&s->codec, VP8D_GET_LAST_REF_UPDATES, &reference_updates) == 0) {
+				if (frame_info.pictureid_present && ((reference_updates & VP8_GOLD_FRAME) || (reference_updates & VP8_ALTR_FRAME))) {
+					vp8rtpfmt_send_rpsi(&s->unpacker, frame_info.pictureid);
+				}
+			}
+
+			if (s->yuv_width != img->d_w || s->yuv_height != img->d_h) {
+				if (s->yuv_msg) freemsg(s->yuv_msg);
+				s->yuv_msg = ms_yuv_buf_alloc(&s->outbuf, img->d_w, img->d_h);
+				ms_message("MSVp8Dec: video is %ix%i", img->d_w, img->d_h);
+				s->yuv_width = img->d_w;
+				s->yuv_height = img->d_h;
+				ms_filter_notify_no_arg(f, MS_FILTER_OUTPUT_FMT_CHANGED);
+			}
+
+			/* scale/copy frame to destination mblk_t */
+			for (i = 0; i < 3; i++) {
+				uint8_t *dest = s->outbuf.planes[i];
+				uint8_t *src = img->planes[i];
+				int h = img->d_h >> ((i > 0) ? 1 : 0);
+
+				for (j = 0; j < h; j++) {
+					memcpy(dest, src, s->outbuf.strides[i]);
+					dest += s->outbuf.strides[i];
+					src += img->stride[i];
+				}
+			}
+			ms_queue_put(f->outputs[0], dupmsg(s->yuv_msg));
+
+			ms_average_fps_update(&s->fps, (uint32_t)f->ticker->time);
+			if (!s->first_image_decoded) {
+				s->first_image_decoded = TRUE;
+				ms_filter_notify_no_arg(f, MS_VIDEO_DECODER_FIRST_IMAGE_DECODED);
 			}
 		}
-		ms_queue_put(f->outputs[0], dupmsg(s->yuv_msg));
 
-		if (ms_average_fps_update(&s->fps, f->ticker->time)) {
-			ms_message("VP8 decoder: Frame size: %dx%d", s->yuv_width, s->yuv_height);
-		}
-		if (!s->first_image_decoded) {
-			s->first_image_decoded = TRUE;
-			ms_filter_notify_no_arg(f, MS_VIDEO_DECODER_FIRST_IMAGE_DECODED);
+		while ((im = ms_queue_get(&mtofree_queue)) != NULL) {
+			freemsg(im);
 		}
 	}
-
-	while ((im = ms_queue_get(&mtofree_queue)) != NULL) {
-		freemsg(im);
-	}
+	
+	ms_filter_unlock(f);
 }
 
 static void dec_postprocess(MSFilter *f) {
@@ -1002,6 +1030,19 @@ static int dec_freeze_on_error(MSFilter *f, void *data) {
 	return 0;
 }
 
+static int dec_reset(MSFilter *f, void *data) {
+	DecState *s = (DecState *)f->data;
+	ms_message("Reseting VP8 decoder");
+	ms_filter_lock(f);
+	vpx_codec_destroy(&s->codec);
+	if(vpx_codec_dec_init(&s->codec, s->iface, NULL, s->flags)){
+		ms_error("Failed to reinitialize VP8 decoder");
+	}
+	s->first_image_decoded = FALSE;
+	ms_filter_unlock(f);
+	return 0;
+}
+
 static int dec_get_vsize(MSFilter *f, void *data) {
 	DecState *s = (DecState *)f->data;
 	MSVideoSize *vsize = (MSVideoSize *)data;
@@ -1032,6 +1073,7 @@ static MSFilterMethod dec_methods[] = {
 	{ MS_VIDEO_DECODER_RESET_FIRST_IMAGE_NOTIFICATION, dec_reset_first_image },
 	{ MS_VIDEO_DECODER_ENABLE_AVPF,                    dec_enable_avpf       },
 	{ MS_VIDEO_DECODER_FREEZE_ON_ERROR,                dec_freeze_on_error   },
+	{ MS_VIDEO_DECODER_RESET,                          dec_reset             },
 	{ MS_FILTER_GET_VIDEO_SIZE,                        dec_get_vsize         },
 	{ MS_FILTER_GET_FPS,                               dec_get_fps           },
 	{ MS_FILTER_GET_OUTPUT_FMT,                        dec_get_out_fmt       },
