@@ -23,6 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
+#include "mediastreamer2/msticker.h"
 #include "layouts.h"
 
 #include <X11/Xlib.h>
@@ -172,7 +173,7 @@ static void x11video_prepare(MSFilter *f){
 	XvPortID port=-1;
 	int imgfmt_id=0;
 	XShmSegmentInfo *shminfo=&s->shminfo;
-	XWindowAttributes wa;
+	XWindowAttributes wa = {0};
 
 	if (s->display==NULL) return;
 	if (s->window_id==0){
@@ -312,6 +313,7 @@ static void x11video_prepare(MSFilter *f){
 static void x11video_unprepare(MSFilter *f){
 	X11Video *s=(X11Video*)f->data;
 	if (s->port!=-1){
+		XvStopVideo(s->display,s->port, s->window_id);
 		XvUngrabPort(s->display,s->port,CurrentTime);
 		s->port=-1;
 	}
@@ -359,6 +361,7 @@ static void x11video_process(MSFilter *f){
 	bool_t precious=FALSE;
 	bool_t local_precious=FALSE;
 	XWindowAttributes wa;
+	MSTickerLateEvent late_info;
 
 	ms_filter_lock(f);
 
@@ -376,11 +379,15 @@ static void x11video_process(MSFilter *f){
 		XClearWindow(obj->display,obj->window_id);
 	}
 
+	ms_ticker_get_last_late_tick(f->ticker, &late_info);
+	if(late_info.current_late_ms > 100) {
+		ms_warning("Dropping frames because we're late");
+		goto end;
+	}
 
 	if (!obj->show) {
 		goto end;
 	}
-	if (!obj->ready) x11video_prepare(f);
 	if (!obj->ready){
 		goto end;
 	}
