@@ -57,32 +57,57 @@ static void disable_checksums(ortp_socket_t sock) {
 #endif
 }
 
+static int _ms_ticker_prio_from_env(const char *penv, MSTickerPrio *prio) {
+	if (strcasecmp(penv, "NORMAL") == 0) {
+		*prio = MS_TICKER_PRIO_NORMAL;
+		return 0;
+	}
+	if (strcasecmp(penv, "HIGH") == 0) {
+		*prio = MS_TICKER_PRIO_HIGH;
+		return 0;
+	}
+	if (strcasecmp(penv, "REALTIME") == 0) {
+		*prio = MS_TICKER_PRIO_REALTIME;
+		return 0;
+	}
+	ms_error("Undefined priority %s", penv);
+	return -1;
+}
+
 MSTickerPrio __ms_get_default_prio(bool_t is_video) {
-	const char *penv;
+	const char *penv = NULL;
+	MSTickerPrio prio;
 
 	if (is_video) {
+#ifndef MS2_WINDOWS_UNIVERSAL
+		penv = getenv("MS_VIDEO_PRIO");
+#endif
+		if(penv && _ms_ticker_prio_from_env(penv, &prio) == 0) return prio;
+
 #ifdef __ios
 		return MS_TICKER_PRIO_HIGH;
 #else
 		return MS_TICKER_PRIO_NORMAL;
 #endif
-	}
-
-	penv = getenv("MS_AUDIO_PRIO");
-	if (penv) {
-		if (strcasecmp(penv, "NORMAL") == 0) return MS_TICKER_PRIO_NORMAL;
-		if (strcasecmp(penv, "HIGH") == 0) return MS_TICKER_PRIO_HIGH;
-		if (strcasecmp(penv, "REALTIME") == 0) return MS_TICKER_PRIO_REALTIME;
-		ms_error("Undefined priority %s", penv);
-	}
-#ifdef __linux
-	return MS_TICKER_PRIO_REALTIME;
-#else
-	return MS_TICKER_PRIO_HIGH;
+	} else {
+#ifndef MS2_WINDOWS_UNIVERSAL
+		penv = getenv("MS_AUDIO_PRIO");
 #endif
+		if (penv && _ms_ticker_prio_from_env(penv, &prio) == 0) return prio;
+
+		return MS_TICKER_PRIO_HIGH;
+
+	}
 }
 
-RtpSession * create_duplex_rtpsession(const char* local_ip, int loc_rtp_port, int loc_rtcp_port) {
+void media_stream_init(MediaStream *stream, MSFactory *factory) {
+	stream->evd = ortp_ev_dispatcher_new(stream->sessions.rtp_session);
+	stream->evq = ortp_ev_queue_new();
+	stream->factory = factory; /*the factory is used later to instanciate everything in mediastreamer2.*/
+	rtp_session_register_event_queue(stream->sessions.rtp_session, stream->evq);
+}
+
+RtpSession * ms_create_duplex_rtp_session(const char* local_ip, int loc_rtp_port, int loc_rtcp_port) {
 	RtpSession *rtpr;
 
 	rtpr = rtp_session_new(RTP_SESSION_SENDRECV);
@@ -145,13 +170,10 @@ void ms_media_stream_sessions_uninit(MSMediaStreamSessions *sessions){
 }
 
 void media_stream_free(MediaStream *stream) {
-	if (stream->sessions.rtp_session != NULL){
-		rtp_session_unregister_event_queue(stream->sessions.rtp_session, stream->evq);
-	}
-	if (stream->owns_sessions){
-		ms_media_stream_sessions_uninit(&stream->sessions);
-	}
-	if (stream->evq) ortp_ev_queue_destroy(stream->evq);
+	if (stream->sessions.rtp_session != NULL) rtp_session_unregister_event_queue(stream->sessions.rtp_session, stream->evq);
+	if (stream->evq != NULL) ortp_ev_queue_destroy(stream->evq);
+	if (stream->evd != NULL) ortp_ev_dispatcher_destroy(stream->evd);
+	if (stream->owns_sessions) ms_media_stream_sessions_uninit(&stream->sessions);
 	if (stream->rc != NULL) ms_bitrate_controller_destroy(stream->rc);
 	if (stream->rtpsend != NULL) ms_filter_destroy(stream->rtpsend);
 	if (stream->rtprecv != NULL) ms_filter_destroy(stream->rtprecv);
@@ -159,7 +181,10 @@ void media_stream_free(MediaStream *stream) {
 	if (stream->decoder != NULL) ms_filter_destroy(stream->decoder);
 	if (stream->voidsink != NULL) ms_filter_destroy(stream->voidsink);
 	if (stream->qi) ms_quality_indicator_destroy(stream->qi);
+}
 
+bool_t media_stream_started(MediaStream *stream) {
+	return stream->start_time != 0;
 }
 
 void media_stream_set_rtcp_information(MediaStream *stream, const char *cname, const char *tool) {
@@ -192,13 +217,20 @@ void media_stream_enable_adaptive_jittcomp(MediaStream *stream, bool_t enabled)
 	rtp_session_enable_adaptive_jitter_compensation(stream->sessions.rtp_session, enabled);
 }
 
+void media_stream_enable_dtls(MediaStream *stream, MSDtlsSrtpParams *params){
+	if (stream->sessions.dtls_context==NULL) {
+		ms_message("Start DTLS media stream context in stream session [%p]", &(stream->sessions));
+		stream->sessions.dtls_context=ms_dtls_srtp_context_new(&(stream->sessions), params);
+	}
+}
+
 bool_t media_stream_dtls_supported(void){
 	return ms_dtls_srtp_available();
 }
 
 /*deprecated*/
 bool_t media_stream_enable_srtp(MediaStream *stream, MSCryptoSuite suite, const char *snd_key, const char *rcv_key) {
-	return media_stream_set_srtp_recv_key_b64(&(stream->sessions),suite,rcv_key)==0 && media_stream_set_srtp_send_key_b64(&(stream->sessions),suite,snd_key)==0;
+	return ms_media_stream_sessions_set_srtp_recv_key_b64(&stream->sessions,suite,rcv_key)==0 && ms_media_stream_sessions_set_srtp_send_key_b64(&stream->sessions,suite,snd_key)==0;
 }
 
 const MSQualityIndicator *media_stream_get_quality_indicator(MediaStream *stream){
@@ -283,6 +315,14 @@ void media_stream_iterate(MediaStream *stream){
 
 	if (stream->rc) ms_bitrate_controller_update(stream->rc);
 
+	if (stream->evd) {
+		ortp_ev_dispatcher_iterate(stream->evd);
+	}
+	
+	if (stream->type == MSText) { // TODO: find a better way
+		text_stream_iterate((TextStream *)stream);
+	}
+
 	if (stream->evq){
 		OrtpEvent *ev=NULL;
 
@@ -295,8 +335,18 @@ void media_stream_iterate(MediaStream *stream){
 				ms_message("%s_stream_iterate[%p], local statistics available:"
 							"\n\tLocal current jitter buffer size: %5.1fms",
 					media_stream_type_str(stream), stream, rtp_session_get_jitter_stats(stream->sessions.rtp_session)->jitter_buffer_size_ms);
-			}else if ((evt==ORTP_EVENT_STUN_PACKET_RECEIVED)&&(stream->ice_check_list)){
-				ice_handle_stun_packet(stream->ice_check_list,stream->sessions.rtp_session,ortp_event_get_data(ev));
+			} else if (evt==ORTP_EVENT_STUN_PACKET_RECEIVED){
+				if (stream->ice_check_list) {
+						ice_handle_stun_packet(stream->ice_check_list,stream->sessions.rtp_session,ortp_event_get_data(ev));
+				} else if (rtp_session_get_symmetric_rtp(stream->sessions.rtp_session)){
+					/*try to know if we can trust stun packets for symetric rtp*/
+					rtp_stats_t stats;
+					media_stream_get_local_rtp_stats(stream, &stats);
+					if (stats.packet_recv == 0 && !ms_is_multicast_addr((const struct sockaddr *)&stream->sessions.rtp_session->rtp.gs.rem_addr)) {
+						memcpy(&stream->sessions.rtp_session->rtp.gs.rem_addr,&ortp_event_get_data(ev)->source_addr,ortp_event_get_data(ev)->source_addrlen);
+						ms_message("stun packet received but no rtp yet for stream [%p], switching rtp destination address",stream);
+					}
+				}
 			} else if ((evt == ORTP_EVENT_ZRTP_ENCRYPTION_CHANGED) || (evt == ORTP_EVENT_DTLS_ENCRYPTION_CHANGED)) {
 				ms_message("%s_stream_iterate[%p]: is %s ",media_stream_type_str(stream) , stream, media_stream_secured(stream) ? "encrypted" : "not encrypted");
 			}
@@ -389,15 +439,12 @@ bool_t media_stream_secured (const MediaStream *stream) {
 
 	switch (stream->type) {
 	case MSAudio:
+	case MSText:
 		/*fixme need also audio stream direction to be more precise*/
-		return media_stream_session_secured(&stream->sessions,MediaStreamSendRecv);
+		return ms_media_stream_sessions_secured(&stream->sessions, MediaStreamSendRecv);
 	case MSVideo:{
 		VideoStream *vs = (VideoStream*)stream;
-		switch (vs->dir) {
-		case VideoStreamSendOnly: return media_stream_session_secured(&stream->sessions,MediaStreamSendOnly);
-		case VideoStreamRecvOnly: return media_stream_session_secured(&stream->sessions,MediaStreamRecvOnly);
-		case VideoStreamSendRecv: return media_stream_session_secured(&stream->sessions,MediaStreamSendRecv);
-		}
+		return ms_media_stream_sessions_secured(&stream->sessions, vs->dir);
 	}
 	}
 	return FALSE;
@@ -480,6 +527,55 @@ int ms_crypto_suite_to_name_params(MSCryptoSuite cs, MSCryptoSuiteNameParams *pa
 	return 0;
 }
 
+OrtpEvDispatcher* media_stream_get_event_dispatcher(const MediaStream *stream) {
+	return stream->evd;
+}
+
+const char *ms_resource_type_to_string(MSResourceType type){
+	switch(type){
+		case MSResourceDefault:
+			return "MSResourceDefault";
+		case MSResourceInvalid:
+			return "MSResourceInvalid";
+		case MSResourceCamera:
+			return "MSResourceCamera";
+		case MSResourceFile:
+			return "MSResourceFile";
+		case MSResourceRtp:
+			return "MSResourceRtp";
+		case MSResourceSoundcard:
+			return "MSResourceSoundcard";
+	}
+	return "INVALID";
+}
+
+bool_t ms_media_resource_is_consistent(const MSMediaResource *r){
+	switch(r->type){
+		case MSResourceCamera:
+		case MSResourceRtp:
+		case MSResourceSoundcard:
+			if (r->resource_arg == NULL){
+				ms_error("No resource argument specified for resource type %s", ms_resource_type_to_string(r->type));
+				return FALSE;
+			}
+			return TRUE;
+		break;
+		case MSResourceFile:
+			/*setting up file player/recorder without specifying the file to play immediately is allowed*/
+		case MSResourceDefault:
+			return TRUE;
+		case MSResourceInvalid:
+			ms_error("Invalid resource type specified");
+			return FALSE;
+	}
+	ms_error("Unsupported media resource type [%i]", (int)r->type);  
+	return FALSE;
+}
+
+bool_t ms_media_stream_io_is_consistent(const MSMediaStreamIO *io){
+	return ms_media_resource_is_consistent(&io->input) && ms_media_resource_is_consistent(&io->output);
+}
+
 /*stubs*/
 #ifndef VIDEO_ENABLED
 void video_stream_open_player(VideoStream *stream, MSFilter *sink){
@@ -491,5 +587,10 @@ void video_stream_close_player(VideoStream *stream){
 const char *video_stream_get_default_video_renderer(void){
 	return NULL;
 }
+
+MSWebCamDesc *ms_mire_webcam_desc_get(void){
+	return NULL;
+}
+
 #endif
 
