@@ -26,10 +26,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2_tester.h"
 #include "mediastreamer2_tester_private.h"
 
-#include <stdio.h>
-#include "CUnit/Basic.h"
-
-
 static RtpProfile rtp_profile;
 
 #define OPUS_PAYLOAD_TYPE    121
@@ -37,7 +33,7 @@ static RtpProfile rtp_profile;
 #define SILK16_PAYLOAD_TYPE  123
 #define PCMA8_PAYLOAD_TYPE 8
 
-static int tester_init(void) {
+static int tester_before_all(void) {
 	ms_init();
 	ms_filter_enable_statistics(TRUE);
 	ortp_init();
@@ -50,7 +46,7 @@ static int tester_init(void) {
 	return 0;
 }
 
-static int tester_cleanup(void) {
+static int tester_after_all(void) {
 	ms_exit();
 	rtp_profile_clear_all(&rtp_profile);
 	return 0;
@@ -72,8 +68,10 @@ static int tester_cleanup(void) {
 #define MULTICAST_IP  "224.1.2.3"
 
 typedef struct _stats_t {
+	OrtpEvQueue *q;
 	rtp_stats_t rtp;
 	int number_of_EndOfFile;
+	int number_of_TMMBR;
 } stats_t;
 
 static void reset_stats(stats_t* s) {
@@ -93,6 +91,32 @@ static void notify_cb(void *user_data, MSFilter *f, unsigned int event, void *ev
 	}
 }
 
+static void event_queue_cb(MediaStream *ms, void *user_pointer) {
+	stats_t *st = (stats_t *)user_pointer;
+	OrtpEvent *ev = NULL;
+
+	if (st->q != NULL) {
+		while ((ev = ortp_ev_queue_get(st->q)) != NULL) {
+			OrtpEventType evt = ortp_event_get_type(ev);
+			OrtpEventData *d = ortp_event_get_data(ev);
+			if (evt == ORTP_EVENT_TMMBR_RECEIVED) {
+				do {
+					if (rtcp_is_RTPFB(d->packet)) {
+						switch (rtcp_RTPFB_get_type(d->packet)) {
+							case RTCP_RTPFB_TMMBR:
+								st->number_of_TMMBR++;
+								break;
+							default:
+								break;
+						}
+					}
+				} while (rtcp_next_packet(d->packet));
+			}
+			ortp_event_destroy(ev);
+		}
+	}
+}
+
 static void basic_audio_stream_base(	const char* marielle_local_ip
 									, 	int marielle_local_rtp_port
 									, 	int marielle_local_rtcp_port
@@ -104,9 +128,9 @@ static void basic_audio_stream_base(	const char* marielle_local_ip
 	AudioStream * 	margaux = audio_stream_new2 (margaux_local_ip, margaux_local_rtp_port,margaux_local_rtcp_port);
 	stats_t margaux_stats;
 	RtpProfile* profile = rtp_profile_new("default profile");
-	char* hello_file = ms_strdup_printf("%s/%s", mediastreamer2_tester_get_file_root(), HELLO_8K_1S_FILE);
-	char* recorded_file = ms_strdup_printf("%s/%s", mediastreamer2_tester_get_writable_dir(), RECORDED_8K_1S_FILE);
-	int dummy=0;
+	char* hello_file = bc_tester_res(HELLO_8K_1S_FILE);
+	char* recorded_file = bc_tester_file(RECORDED_8K_1S_FILE);
+	int marielle_rtp_sent=0;
 	rtp_session_set_multicast_loopback(marielle->ms.sessions.rtp_session,TRUE);
 	rtp_session_set_multicast_loopback(margaux->ms.sessions.rtp_session,TRUE);
 
@@ -116,7 +140,7 @@ static void basic_audio_stream_base(	const char* marielle_local_ip
 	rtp_profile_set_payload (profile,0,&payload_type_pcmu8000);
 
 
-	CU_ASSERT_EQUAL(audio_stream_start_full(margaux
+	BC_ASSERT_EQUAL(audio_stream_start_full(margaux
 											, profile
 											, ms_is_multicast(margaux_local_ip)?margaux_local_ip:marielle_local_ip
 											, ms_is_multicast(margaux_local_ip)?margaux_local_rtp_port:marielle_local_rtp_port
@@ -128,9 +152,10 @@ static void basic_audio_stream_base(	const char* marielle_local_ip
 											, recorded_file
 											, NULL
 											, NULL
-											, 0),0);
+											, 0)
+					,0, int, "%d");
 
-	CU_ASSERT_EQUAL(audio_stream_start_full(marielle
+	BC_ASSERT_EQUAL(audio_stream_start_full(marielle
 											, profile
 											, margaux_local_ip
 											, margaux_local_rtp_port
@@ -142,27 +167,26 @@ static void basic_audio_stream_base(	const char* marielle_local_ip
 											, NULL
 											, NULL
 											, NULL
-											, 0),0);
+											, 0)
+					,0, int, "%d");
 
 	ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats,TRUE);
 
-	CU_ASSERT_TRUE(wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,1,12000));
-
-	/*make sure packets can cross from sender to receiver*/
-	wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,500);
+	wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,1,12000);
 
 	audio_stream_get_local_rtp_stats(marielle,&marielle_stats.rtp);
 	audio_stream_get_local_rtp_stats(margaux,&margaux_stats.rtp);
-
-	/* No packet loss is assumed */
-	CU_ASSERT_EQUAL(marielle_stats.rtp.sent,margaux_stats.rtp.recv);
+	marielle_rtp_sent = marielle_stats.rtp.sent;
 
 	audio_stream_stop(marielle);
+	/* No packet loss is assumed */
+	wait_for_until(&margaux->ms,NULL,(int*)&margaux_stats.rtp.hw_recv,marielle_rtp_sent,2500);
+
 	audio_stream_stop(margaux);
 
 	unlink(recorded_file);
-	ms_free(recorded_file);
-	ms_free(hello_file);
+	free(recorded_file);
+	free(hello_file);
 	rtp_profile_destroy(profile);
 }
 
@@ -184,13 +208,13 @@ static void encrypted_audio_stream_base( bool_t change_ssrc,
 	AudioStream * 	marielle = audio_stream_new (MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT,FALSE);
 	AudioStream * 	margaux = audio_stream_new (MARGAUX_RTP_PORT,MARGAUX_RTCP_PORT, FALSE);
 	RtpProfile* profile = rtp_profile_new("default profile");
-	char* hello_file = ms_strdup_printf("%s/%s", mediastreamer2_tester_get_file_root(), HELLO_8K_1S_FILE);
-	char* recorded_file = ms_strdup_printf("%s/%s", mediastreamer2_tester_get_writable_dir(), RECORDED_8K_1S_FILE);
+	char* hello_file = bc_tester_res(HELLO_8K_1S_FILE);
+	char* recorded_file = bc_tester_file(RECORDED_8K_1S_FILE);
 	stats_t marielle_stats;
 	stats_t margaux_stats;
 	int dummy=0;
 	int number_of_dropped_packets=0;
-	media_stream_session_encryption_mandatory_enable(&marielle->ms.sessions,encryption_mandatory);
+	ms_media_stream_sessions_set_encryption_mandatory(&marielle->ms.sessions,encryption_mandatory);
 
 	if (ms_srtp_supported()) {
 		reset_stats(&marielle_stats);
@@ -198,80 +222,82 @@ static void encrypted_audio_stream_base( bool_t change_ssrc,
 
 		rtp_profile_set_payload (profile,0,&payload_type_pcmu8000);
 
-		CU_ASSERT_EQUAL(audio_stream_start_full(margaux
-				, profile
-				, MARIELLE_IP
-				, MARIELLE_RTP_PORT
-				, MARIELLE_IP
-				, MARIELLE_RTCP_PORT
-				, 0
-				, 50
-				, NULL
-				, recorded_file
-				, NULL
-				, NULL
-				, 0),0);
+		BC_ASSERT_EQUAL(audio_stream_start_full(margaux
+												, profile
+												, MARIELLE_IP
+												, MARIELLE_RTP_PORT
+												, MARIELLE_IP
+												, MARIELLE_RTCP_PORT
+												, 0
+												, 50
+												, NULL
+												, recorded_file
+												, NULL
+												, NULL
+												, 0)
+		,0, int, "%d");
 
-		CU_ASSERT_EQUAL(audio_stream_start_full(marielle
-				, profile
-				, MARGAUX_IP
-				, MARGAUX_RTP_PORT
-				, MARGAUX_IP
-				, MARGAUX_RTCP_PORT
-				, 0
-				, 50
-				, hello_file
-				, NULL
-				, NULL
-				, NULL
-				, 0),0);
+		BC_ASSERT_EQUAL(audio_stream_start_full(marielle
+												, profile
+												, MARGAUX_IP
+												, MARGAUX_RTP_PORT
+												, MARGAUX_IP
+												, MARGAUX_RTCP_PORT
+												, 0
+												, 50
+												, hello_file
+												, NULL
+												, NULL
+												, NULL
+												, 0)
+		,0, int, "%d");
 
 		if (encryption_mandatory) {
 			/*wait a bit to make sure packets are discarded*/
 			wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,1000);
 			audio_stream_get_local_rtp_stats(margaux,&margaux_stats.rtp);
 			audio_stream_get_local_rtp_stats(marielle,&marielle_stats.rtp);
-			CU_ASSERT_EQUAL(margaux_stats.rtp.recv,0);
+			BC_ASSERT_EQUAL(margaux_stats.rtp.recv,0, int, "%d");
 			number_of_dropped_packets=marielle_stats.rtp.packet_sent;
 		}
 
 		if (send_key_first) {
-			CU_ASSERT_TRUE(media_stream_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") == 0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") == 0);
 			if (set_both_send_recv_key)
-				CU_ASSERT_TRUE(media_stream_set_srtp_send_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") == 0);
+				BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_send_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") == 0);
 
-			CU_ASSERT_TRUE(media_stream_set_srtp_recv_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") ==0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_recv_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") ==0);
 			if (set_both_send_recv_key)
-				CU_ASSERT_TRUE(media_stream_set_srtp_recv_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") ==0);
+				BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_recv_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") ==0);
 
 		} else {
-			CU_ASSERT_TRUE(media_stream_set_srtp_recv_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") ==0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_recv_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") ==0);
 			if (set_both_send_recv_key)
-				CU_ASSERT_TRUE(media_stream_set_srtp_recv_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") ==0);
+				BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_recv_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") ==0);
 
-			CU_ASSERT_TRUE(media_stream_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") == 0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") == 0);
 			if (set_both_send_recv_key)
-				CU_ASSERT_TRUE(media_stream_set_srtp_send_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") == 0);
+				BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_send_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "6jCLmtRkVW9E/BUuJtYj/R2z6+4iEe06/DWohQ9F") == 0);
 
 		}
 
 		if (set_both_send_recv_key) {
 			wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,1000);
-			CU_ASSERT_TRUE(media_stream_secured((MediaStream*)marielle));
-			CU_ASSERT_TRUE(media_stream_secured((MediaStream*)margaux));
+			BC_ASSERT_TRUE(media_stream_secured((MediaStream*)marielle));
+			BC_ASSERT_TRUE(media_stream_secured((MediaStream*)margaux));
 		} else {
 			/*so far, not possible to know audio stream direction*/
-			CU_ASSERT_FALSE(media_stream_secured((MediaStream*)marielle));
-			CU_ASSERT_FALSE(media_stream_secured((MediaStream*)margaux));
+			BC_ASSERT_FALSE(media_stream_secured((MediaStream*)marielle));
+			BC_ASSERT_FALSE(media_stream_secured((MediaStream*)margaux));
 		}
 
 		ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats,TRUE);
 		if (change_send_key_in_the_middle) {
 			wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,2000);
-			CU_ASSERT_TRUE(media_stream_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "eCYF4nYyCvmCpFWjUeDaxI2GWp2BzCRlIPfg52Te") == 0);
-			CU_ASSERT_TRUE(media_stream_set_srtp_recv_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "eCYF4nYyCvmCpFWjUeDaxI2GWp2BzCRlIPfg52Te") ==0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "eCYF4nYyCvmCpFWjUeDaxI2GWp2BzCRlIPfg52Te") == 0);
+			BC_ASSERT_TRUE(ms_media_stream_sessions_set_srtp_recv_key_b64(&(margaux->ms.sessions), MS_AES_128_SHA1_32, "eCYF4nYyCvmCpFWjUeDaxI2GWp2BzCRlIPfg52Te") ==0);
 		}
-		CU_ASSERT_TRUE(wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,1,12000));
+		BC_ASSERT_TRUE(wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,1,12000));
 
 		/*make sure packets can cross from sender to receiver*/
 		wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,500);
@@ -282,31 +308,32 @@ static void encrypted_audio_stream_base( bool_t change_ssrc,
 		/* No packet loss is assumed */
 		if (change_send_key_in_the_middle) {
 			/*we can accept one or 2 error in such case*/
-			CU_ASSERT_TRUE((marielle_stats.rtp.packet_sent-margaux_stats.rtp.packet_recv-number_of_dropped_packets)<3);
+			BC_ASSERT_TRUE((marielle_stats.rtp.packet_sent-margaux_stats.rtp.packet_recv-number_of_dropped_packets)<3);
 		} else
-			CU_ASSERT_EQUAL(marielle_stats.rtp.packet_sent,margaux_stats.rtp.packet_recv+number_of_dropped_packets);
+			BC_ASSERT_EQUAL(marielle_stats.rtp.packet_sent,margaux_stats.rtp.packet_recv+number_of_dropped_packets, int, "%d");
 
 		if (change_ssrc) {
 			audio_stream_stop(marielle);
 			marielle = audio_stream_new (MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT,FALSE);
-			CU_ASSERT_EQUAL(audio_stream_start_full(marielle
-							, profile
-							, MARGAUX_IP
-							, MARGAUX_RTP_PORT
-							, MARGAUX_IP
-							, MARGAUX_RTCP_PORT
-							, 0
-							, 50
-							, hello_file
-							, NULL
-							, NULL
-							, NULL
-							, 0),0);
-			CU_ASSERT_FATAL(media_stream_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") == 0);
+			BC_ASSERT_EQUAL(audio_stream_start_full(marielle
+													, profile
+													, MARGAUX_IP
+													, MARGAUX_RTP_PORT
+													, MARGAUX_IP
+													, MARGAUX_RTCP_PORT
+													, 0
+													, 50
+													, hello_file
+													, NULL
+													, NULL
+													, NULL
+													, 0)
+			,0, int, "%d");
+			BC_ASSERT_FATAL(ms_media_stream_sessions_set_srtp_send_key_b64(&(marielle->ms.sessions), MS_AES_128_SHA1_32, "d0RmdmcmVCspeEc3QGZiNWpVLFJhQX1cfHAwJSoj") == 0);
 
 			ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats,TRUE);
 
-			CU_ASSERT_TRUE(wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,2,12000));
+			BC_ASSERT_TRUE(wait_for_until(&marielle->ms,&margaux->ms,&marielle_stats.number_of_EndOfFile,2,12000));
 
 			/*make sure packets can cross from sender to receiver*/
 			wait_for_until(&marielle->ms,&margaux->ms,&dummy,1,500);
@@ -315,13 +342,13 @@ static void encrypted_audio_stream_base( bool_t change_ssrc,
 			audio_stream_get_local_rtp_stats(margaux,&margaux_stats.rtp);
 
 			/* No packet loss is assumed */
-			CU_ASSERT_EQUAL(marielle_stats.rtp.sent*2,margaux_stats.rtp.recv);
+			BC_ASSERT_EQUAL(marielle_stats.rtp.sent*2,margaux_stats.rtp.recv, int, "%d");
 
 		}
 
 		unlink(recorded_file);
-		ms_free(recorded_file);
-		ms_free(hello_file);
+		free(recorded_file);
+		free(hello_file);
 	} else {
 		ms_warning("srtp not available, skiping...");
 	}
@@ -363,8 +390,8 @@ static void codec_change_for_audio_stream(void) {
 	AudioStream *margaux = audio_stream_new2(MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_RTCP_PORT);
 	stats_t margaux_stats;
 	RtpProfile *profile = rtp_profile_new("default profile");
-	char* hello_file = ms_strdup_printf("%s/%s", mediastreamer2_tester_get_file_root(), HELLO_8K_1S_FILE);
-	char* recorded_file = ms_strdup_printf("%s/%s", mediastreamer2_tester_get_writable_dir(), RECORDED_8K_1S_FILE);
+	char* hello_file = bc_tester_res(HELLO_8K_1S_FILE);
+	char* recorded_file = bc_tester_file(RECORDED_8K_1S_FILE);
 	uint64_t marielle_rtp_sent = 0;
 	int dummy=0;
 
@@ -374,15 +401,15 @@ static void codec_change_for_audio_stream(void) {
 	rtp_profile_set_payload(profile, 0, &payload_type_pcmu8000);
 	rtp_profile_set_payload(profile, 8, &payload_type_pcma8000);
 
-	CU_ASSERT_EQUAL(audio_stream_start_full(margaux, profile, MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_IP, MARIELLE_RTCP_PORT,
-		0, 50, NULL, recorded_file, NULL, NULL, 0), 0);
+	BC_ASSERT_EQUAL(audio_stream_start_full(margaux, profile, MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_IP, MARIELLE_RTCP_PORT,
+		0, 50, NULL, recorded_file, NULL, NULL, 0), 0, int, "%d");
 
-	CU_ASSERT_EQUAL(audio_stream_start_full(marielle, profile, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_IP, MARGAUX_RTCP_PORT,
-		0, 50, hello_file, NULL, NULL, NULL, 0), 0);
+	BC_ASSERT_EQUAL(audio_stream_start_full(marielle, profile, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_IP, MARGAUX_RTCP_PORT,
+		0, 50, hello_file, NULL, NULL, NULL, 0), 0, int, "%d");
 
 	ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats, TRUE);
 
-	CU_ASSERT_TRUE(wait_for_until(&marielle->ms, &margaux->ms, &marielle_stats.number_of_EndOfFile, 1, 12000));
+	BC_ASSERT_TRUE(wait_for_until(&marielle->ms, &margaux->ms, &marielle_stats.number_of_EndOfFile, 1, 12000));
 
 	/*make sure packets can cross from sender to receiver*/
 	wait_for_until(&marielle->ms, &margaux->ms, &dummy, 1, 500);
@@ -391,19 +418,19 @@ static void codec_change_for_audio_stream(void) {
 	audio_stream_get_local_rtp_stats(margaux, &margaux_stats.rtp);
 
 	/* No packet loss is assumed */
-	CU_ASSERT_EQUAL(marielle_stats.rtp.sent, margaux_stats.rtp.recv);
+	BC_ASSERT_EQUAL(marielle_stats.rtp.sent, margaux_stats.rtp.recv, int, "%d");
 	marielle_rtp_sent = marielle_stats.rtp.sent;
 
 	audio_stream_stop(marielle);
 	reset_stats(&marielle_stats);
 	reset_stats(&margaux_stats);
 	marielle = audio_stream_new2(MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT);
-	CU_ASSERT_EQUAL(audio_stream_start_full(marielle, profile, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_IP, MARGAUX_RTCP_PORT,
-		8, 50, hello_file, NULL, NULL, NULL, 0), 0);
+	BC_ASSERT_EQUAL(audio_stream_start_full(marielle, profile, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_IP, MARGAUX_RTCP_PORT,
+		8, 50, hello_file, NULL, NULL, NULL, 0), 0, int, "%d");
 
 	ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats, TRUE);
 
-	CU_ASSERT_TRUE(wait_for_until(&marielle->ms, &margaux->ms, &marielle_stats.number_of_EndOfFile, 1, 12000));
+	BC_ASSERT_TRUE(wait_for_until(&marielle->ms, &margaux->ms, &marielle_stats.number_of_EndOfFile, 1, 12000));
 
 	/*make sure packets can cross from sender to receiver*/
 	wait_for_until(&marielle->ms, &margaux->ms, &dummy, 1, 500);
@@ -412,14 +439,76 @@ static void codec_change_for_audio_stream(void) {
 	audio_stream_get_local_rtp_stats(margaux, &margaux_stats.rtp);
 
 	/* No packet loss is assumed */
-	CU_ASSERT_EQUAL(marielle_stats.rtp.sent + marielle_rtp_sent, margaux_stats.rtp.recv);
-	CU_ASSERT_EQUAL(strcasecmp(margaux->ms.decoder->desc->enc_fmt, "pcma"), 0);
+	BC_ASSERT_EQUAL(marielle_stats.rtp.sent + marielle_rtp_sent, margaux_stats.rtp.recv, int, "%d");
+	BC_ASSERT_EQUAL(strcasecmp(margaux->ms.decoder->desc->enc_fmt, "pcma"), 0, int, "%d");
 	audio_stream_stop(marielle);
 	audio_stream_stop(margaux);
 
 	unlink(recorded_file);
-	ms_free(recorded_file);
-	ms_free(hello_file);
+	free(recorded_file);
+	free(hello_file);
+	rtp_profile_destroy(profile);
+}
+
+static void tmmbr_feedback_for_audio_stream(void) {
+	AudioStream *marielle = audio_stream_new2(MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_RTCP_PORT);
+	stats_t marielle_stats;
+	AudioStream *margaux = audio_stream_new2(MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_RTCP_PORT);
+	stats_t margaux_stats;
+	RtpProfile *profile = rtp_profile_new("default profile");
+	RtpSession *marielle_session;
+	RtpSession *margaux_session;
+	char* hello_file = bc_tester_res(HELLO_8K_1S_FILE);
+	int dummy=0;
+
+	reset_stats(&marielle_stats);
+	reset_stats(&margaux_stats);
+
+	rtp_profile_set_payload(profile, 0, &payload_type_pcmu8000);
+
+	/* Activate AVPF and TMBRR. */
+	payload_type_set_flag(&payload_type_pcmu8000, PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED);
+	marielle_session = audio_stream_get_rtp_session(marielle);
+	rtp_session_enable_avpf_feature(marielle_session, ORTP_AVPF_FEATURE_TMMBR, TRUE);
+	marielle_stats.q = ortp_ev_queue_new();
+	rtp_session_register_event_queue(marielle->ms.sessions.rtp_session, marielle_stats.q);
+	margaux_session = audio_stream_get_rtp_session(margaux);
+	rtp_session_enable_avpf_feature(margaux_session, ORTP_AVPF_FEATURE_TMMBR, TRUE);
+	margaux_stats.q = ortp_ev_queue_new();
+	rtp_session_register_event_queue(margaux->ms.sessions.rtp_session, margaux_stats.q);
+
+	BC_ASSERT_EQUAL(audio_stream_start_full(margaux, profile, MARIELLE_IP, MARIELLE_RTP_PORT, MARIELLE_IP, MARIELLE_RTCP_PORT,
+		0, 50, hello_file, NULL, NULL, NULL, 0), 0, int, "%d");
+
+	BC_ASSERT_EQUAL(audio_stream_start_full(marielle, profile, MARGAUX_IP, MARGAUX_RTP_PORT, MARGAUX_IP, MARGAUX_RTCP_PORT,
+		0, 50, hello_file, NULL, NULL, NULL, 0), 0, int, "%d");
+
+	ms_filter_add_notify_callback(margaux->soundread, notify_cb, &margaux_stats, TRUE);
+	ms_filter_add_notify_callback(marielle->soundread, notify_cb, &marielle_stats, TRUE);
+
+	/* Wait for 1s so that some RTP packets are exchanged before sending the TMMBR. */
+	wait_for_until(&margaux->ms, &marielle->ms, &dummy, 1, 500);
+
+	rtp_session_send_rtcp_fb_tmmbr(margaux_session, 100000);
+	rtp_session_send_rtcp_fb_tmmbr(marielle_session, 200000);
+
+	BC_ASSERT_TRUE(wait_for_until(&margaux->ms, &marielle->ms, &margaux_stats.number_of_EndOfFile, 1, 12000));
+	BC_ASSERT_TRUE(wait_for_until(&marielle->ms, &margaux->ms, &marielle_stats.number_of_EndOfFile, 1, 12000));
+
+	BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->ms, &margaux->ms, &marielle_stats.number_of_TMMBR, 1, 100, event_queue_cb, &marielle_stats, event_queue_cb, &margaux_stats));
+	BC_ASSERT_TRUE(wait_for_until_with_parse_events(&margaux->ms, &marielle->ms, &margaux_stats.number_of_TMMBR, 1, 100, event_queue_cb, &margaux_stats, event_queue_cb, &marielle_stats));
+	BC_ASSERT_EQUAL(marielle_stats.number_of_TMMBR, 1, int, "%d");
+	BC_ASSERT_EQUAL(margaux_stats.number_of_TMMBR, 1, int, "%d");
+
+	/*make sure packets can cross from sender to receiver*/
+	wait_for_until(&marielle->ms, &margaux->ms, &dummy, 1, 500);
+
+	audio_stream_stop(marielle);
+	audio_stream_stop(margaux);
+
+	free(hello_file);
+	ortp_ev_queue_destroy(marielle_stats.q);
+	ortp_ev_queue_destroy(margaux_stats.q);
 	rtp_profile_destroy(profile);
 }
 
@@ -454,7 +543,7 @@ static void audio_stream_dtmf(int codec_payload, int initial_bitrate,int target_
 	marielle_send_bw=media_stream_get_up_bw(&marielle->stream->ms);
 	recv_send_bw_ratio=params.max_bandwidth/marielle_send_bw;
 	ms_message("marielle sent bw= [%f] , target was [%f] recv/send [%f]",marielle_send_bw,params.max_bandwidth,recv_send_bw_ratio);
-	CU_ASSERT_TRUE(recv_send_bw_ratio>0.9);
+	BC_ASSERT_TRUE(recv_send_bw_ratio>0.9);
 
 	stream_manager_delete(marielle);
 	stream_manager_delete(margaux);
@@ -473,13 +562,16 @@ static test_t tests[] = {
 	{ "Encrypted audio stream with key change", encrypted_audio_stream_with_key_change },
 	{ "Encrypted audio stream, encryption mandatory", encrypted_audio_stream_encryption_mandatory },
 	{ "Encrypted audio stream with key change + encryption mandatory", encrypted_audio_stream_with_key_change_encryption_mandatory},
-	{ "Codec change for audio stream", codec_change_for_audio_stream }
+	{ "Codec change for audio stream", codec_change_for_audio_stream },
+	{ "TMMBR feedback for audio stream", tmmbr_feedback_for_audio_stream }
 };
 
 test_suite_t audio_stream_test_suite = {
 	"AudioStream",
-	tester_init,
-	tester_cleanup,
+	tester_before_all,
+	tester_after_all,
+	NULL,
+	NULL,
 	sizeof(tests) / sizeof(tests[0]),
 	tests
 };
